@@ -1,6 +1,7 @@
-/* gtk-playlist.c
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* totem-playlist.c
 
-   Copyright (C) 2002, 2003 Bastien Nocera
+   Copyright (C) 2002, 2003, 2004, 2005 Bastien Nocera
 
    The Gnome Library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public License as
@@ -32,6 +33,7 @@
 #include <libgnomevfs/gnome-vfs-mime-utils.h>
 #include <string.h>
 
+#include "totem-interface.h"
 #include "totem-pl-parser.h"
 #include "debug.h"
 
@@ -165,23 +167,19 @@ static void
 totem_playlist_error (char *title, char *reason, TotemPlaylist *playlist)
 {
 	GtkWidget *error_dialog;
-	char *title_esc, *reason_esc;
-
-	title_esc = g_markup_escape_text (title, -1);
-	reason_esc = g_markup_escape_text (reason, -1);
 
 	error_dialog =
 		gtk_message_dialog_new (GTK_WINDOW (playlist),
 				GTK_DIALOG_MODAL,
 				GTK_MESSAGE_ERROR,
 				GTK_BUTTONS_OK,
-				"<b>%s</b>\n%s.", title_esc, reason_esc);
-	g_free (title_esc);
-	g_free (reason_esc);
+				title);
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (error_dialog),
+						  reason);
+
 	gtk_container_set_border_width (GTK_CONTAINER (error_dialog), 5);
 	gtk_dialog_set_default_response (GTK_DIALOG (error_dialog),
 			GTK_RESPONSE_OK);
-	gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (error_dialog)->label), TRUE);
 	g_signal_connect (G_OBJECT (error_dialog), "destroy", G_CALLBACK
 			(gtk_widget_destroy), error_dialog);
 	g_signal_connect (G_OBJECT (error_dialog), "response", G_CALLBACK
@@ -277,7 +275,7 @@ totem_playlist_create_full_path (const char *path)
 
 static void
 totem_playlist_save_get_iter_func (GtkTreeModel *model,
-		GtkTreeIter *iter, char **uri, char **title)
+		GtkTreeIter *iter, char **uri, char **title, gpointer user_data)
 {
 	gtk_tree_model_get (model, iter,
 			URI_COL, uri,
@@ -293,8 +291,8 @@ totem_playlist_save_current_playlist (TotemPlaylist *playlist, const char *outpu
 
 	retval = totem_pl_parser_write (playlist->_priv->parser,
 			playlist->_priv->model,
-			totem_playlist_save_get_iter_func,
-			output, TOTEM_PL_PARSER_PLS, &error);
+                        totem_playlist_save_get_iter_func,
+			output, TOTEM_PL_PARSER_PLS, NULL, &error);
 
 	if (retval == FALSE)
 	{
@@ -851,8 +849,11 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 				 GTK_DIALOG_MODAL,
 				 GTK_MESSAGE_QUESTION,
 				 GTK_BUTTONS_NONE,
-				 _("A file named '%s' already exists.\nAre you sure you want to overwrite it?"),
-				 filename);
+				 _("Overwrite file?"));
+			gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+								  _("A file named '%s' already exists.  Are you sure you want to overwrite it?"),
+								  filename);
+
 			gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
 			gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 					GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
@@ -1004,6 +1005,7 @@ totem_playlist_down_files (GtkWidget *widget, TotemPlaylist *playlist)
 static int
 totem_playlist_key_press (GtkWidget *win, GdkEventKey *event, TotemPlaylist *playlist)
 {
+	//FIXME add support for GDK_Menu and Ctrl+F10
 	/* Special case some shortcuts */
 	if (event->state != 0
 			&& (event->state &GDK_CONTROL_MASK))
@@ -1028,6 +1030,9 @@ totem_playlist_key_press (GtkWidget *win, GdkEventKey *event, TotemPlaylist *pla
 	{
 		totem_playlist_remove_files (NULL, playlist);
 		return TRUE;
+	} else if (event->keyval == GDK_Escape) {
+		gtk_dialog_response (GTK_DIALOG (playlist),
+				GTK_RESPONSE_CLOSE);
 	}
 
 	return FALSE;
@@ -1061,7 +1066,12 @@ treeview_row_changed (GtkTreeView *treeview, GtkTreePath *arg1,
 {
 	if (totem_playlist_gtk_tree_path_equals
 			(arg1, playlist->_priv->current) != FALSE)
+	{
+		g_signal_emit (G_OBJECT (playlist),
+				totem_playlist_table_signals[CHANGED], 0,
+				NULL);
 		return;
+	}
 
 	if (playlist->_priv->current != NULL)
 	{
@@ -1453,16 +1463,17 @@ totem_playlist_realize (GtkWidget *widget)
 }
 
 GtkWidget*
-totem_playlist_new (const char *glade_filename, GdkPixbuf *playing_pix)
+totem_playlist_new (void)
 {
 	TotemPlaylist *playlist;
 	GtkWidget *container, *item;
-
-	g_return_val_if_fail (glade_filename != NULL, NULL);
+	char *filename;
 
 	playlist = TOTEM_PLAYLIST (g_object_new (GTK_TYPE_PLAYLIST, NULL));
 
-	playlist->_priv->xml = glade_xml_new (glade_filename, NULL, NULL);
+	playlist->_priv->xml = totem_interface_load ("playlist.glade",
+			"playlist", TRUE, NULL);
+
 	if (playlist->_priv->xml == NULL)
 	{
 		totem_playlist_finalize (G_OBJECT (playlist));
@@ -1532,7 +1543,12 @@ totem_playlist_new (const char *glade_filename, GdkPixbuf *playing_pix)
 	/* The configuration */
 	init_config (playlist);
 
-	playlist->_priv->icon = playing_pix;
+	playlist->_priv->icon =
+		totem_interface_load_pixbuf ("playlist-playing.png");
+
+	filename = totem_interface_get_full_path ("playlist-24.png");
+	gtk_window_set_icon_from_file (GTK_WINDOW (playlist), filename, NULL);
+	g_free (filename);
 
 	gtk_widget_show_all (GTK_DIALOG (playlist)->vbox);
 
@@ -1622,35 +1638,18 @@ totem_playlist_clear (TotemPlaylist *playlist)
 	playlist->_priv->current = NULL;
 }
 
-static gboolean
-totem_playlist_current_has_prefix (TotemPlaylist *playlist,
-		const char *prefix)
-{
-	char *current;
-	gboolean retval;
-
-	current = totem_playlist_get_current_mrl (playlist);
-	retval = g_str_has_prefix (current, prefix);
-	g_free (current);
-	return retval;
-}
-
-void
-totem_playlist_clear_with_prefix (TotemPlaylist *playlist, const char *prefix)
+static void
+totem_playlist_clear_with_compare (TotemPlaylist *playlist, GCompareFunc func,
+		gconstpointer data)
 {
 	GList *list = NULL, *l;
 	guint num_items, i;
+	gboolean has_items;
 
 	num_items = PL_LEN;
 	if (num_items == 0)
 		return;
 
-	if (totem_playlist_current_has_prefix (playlist, prefix) == FALSE)
-	{
-		g_warning ("totem_playlist_clear_with_prefix called while current mrl doesn't have prefix '%s'", prefix);
-		return;
-	}
-
 	for (i = 0; i < num_items; i++)
 	{
 		GtkTreeIter iter;
@@ -1670,7 +1669,7 @@ totem_playlist_clear_with_prefix (TotemPlaylist *playlist, const char *prefix)
 		gtk_tree_model_get (playlist->_priv->model, &iter,
 				URI_COL, &mrl, -1);
 
-		if (g_str_has_prefix (mrl, prefix) != FALSE)
+		if ((* func) (mrl, data) != FALSE)
 		{
 			GtkTreePath *path;
 			GtkTreeRowReference *ref;
@@ -1685,6 +1684,7 @@ totem_playlist_clear_with_prefix (TotemPlaylist *playlist, const char *prefix)
 		g_free (mrl);
 	}
 
+	has_items = (list != NULL);
 
 	for (l = list; l != NULL; l = l->next)
 	{
@@ -1700,14 +1700,71 @@ totem_playlist_clear_with_prefix (TotemPlaylist *playlist, const char *prefix)
 	}
 	g_list_free (list);
 
-	playlist->_priv->current_shuffled = -1;
+	if (has_items != FALSE) {
+		playlist->_priv->current_shuffled = -1;
 
-	ensure_shuffled (playlist, playlist->_priv->shuffle);
-	gtk_tree_path_free (playlist->_priv->current);
-	playlist->_priv->current = NULL;
-	g_signal_emit (G_OBJECT (playlist),
-			totem_playlist_table_signals[CURRENT_REMOVED],
-			0, NULL);
+		ensure_shuffled (playlist, playlist->_priv->shuffle);
+		gtk_tree_path_free (playlist->_priv->current);
+		playlist->_priv->current = NULL;
+		g_signal_emit (G_OBJECT (playlist),
+				totem_playlist_table_signals[CURRENT_REMOVED],
+				0, NULL);
+	}
+}
+
+static int
+totem_playlist_compare_with_prefix (gconstpointer a, gconstpointer b)
+{
+	const char *mrl = (const char *) a;
+	const char *prefix = (const char *) b;
+
+	return g_str_has_prefix (mrl, prefix);
+}
+
+void
+totem_playlist_clear_with_prefix (TotemPlaylist *playlist, const char *prefix)
+{
+	totem_playlist_clear_with_compare (playlist,
+			(GCompareFunc) totem_playlist_compare_with_prefix,
+			(void *)prefix);
+}
+
+static int
+totem_playlist_compare_with_volume (gpointer a, gpointer b)
+{
+	GnomeVFSVolume *clear_volume = (GnomeVFSVolume *) b;
+	const char *mrl = (const char *) a;
+
+	char *filename;
+	GnomeVFSVolume *volume;
+	GnomeVFSVolumeMonitor *monitor;
+	gboolean retval = FALSE;
+
+	if (g_str_has_prefix (mrl, "file:///") == FALSE)
+		return FALSE;
+
+	monitor = gnome_vfs_get_volume_monitor ();
+
+	filename = g_filename_from_uri (mrl, NULL, NULL);
+	volume = gnome_vfs_volume_monitor_get_volume_for_path
+		(monitor, filename);
+	g_free (filename);
+
+	if (volume == clear_volume)
+		retval = TRUE;
+
+	gnome_vfs_volume_unref (volume);
+
+	return retval;
+}
+
+void
+totem_playlist_clear_with_gnome_vfs_volume (TotemPlaylist *playlist,
+		GnomeVFSVolume *volume)
+{
+	totem_playlist_clear_with_compare (playlist,
+			(GCompareFunc) totem_playlist_compare_with_volume,
+			volume);
 }
 
 char
@@ -2034,6 +2091,46 @@ totem_playlist_set_at_end (TotemPlaylist *playlist)
 	}
 }
 
+guint
+totem_playlist_get_current (TotemPlaylist *playlist)
+{
+	char *path;
+	double index;
+
+	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), 0);
+
+	path = gtk_tree_path_to_string (playlist->_priv->current);
+	if (path == NULL)
+		return -1;
+
+	index = g_ascii_strtod (path, NULL);
+	g_free (path);
+
+	return index;
+}
+
+guint
+totem_playlist_get_last (TotemPlaylist *playlist)
+{
+	g_return_val_if_fail (GTK_IS_PLAYLIST (playlist), -1);
+
+	return PL_LEN - 1;
+}
+
+void
+totem_playlist_set_current (TotemPlaylist *playlist, guint index)
+{
+	g_return_if_fail (GTK_IS_PLAYLIST (playlist));
+
+	if (index >= (guint) PL_LEN)
+		return;
+
+	totem_playlist_unset_playing (playlist);
+	//FIXME problems when shuffled?
+	gtk_tree_path_free (playlist->_priv->current);
+	playlist->_priv->current = gtk_tree_path_new_from_indices (index, -1);
+}
+
 static void
 totem_playlist_class_init (TotemPlaylistClass *klass)
 {
