@@ -21,7 +21,9 @@
 #define __TOTEM_PRIVATE_H__
 
 #include <glade/glade.h>
-#include <glib/gi18n.h>
+#include <gconf/gconf-client.h>
+#include <libgnomevfs/gnome-vfs.h>
+
 #include "totem-remote.h"
 #include "scrsaver.h"
 #include "egg-recent-model.h"
@@ -29,6 +31,7 @@
 #include "totem-playlist.h"
 #include "bacon-message-connection.h"
 #include "bacon-video-widget.h"
+#include "totem-skipto.h"
 
 typedef enum {
 	TOTEM_CONTROLS_VISIBLE,
@@ -48,9 +51,13 @@ struct Totem {
 	GtkWidget *win;
 	BaconVideoWidget *bvw;
 	GtkWidget *prefs;
-	GtkWidget *properties;
 	GtkWidget *statusbar;
 
+	/* Separate Dialogs */
+	GtkWidget *properties;
+	TotemSkipto *skipto;
+	GtkWidget *about;
+
 	/* Play/Pause */
 	GtkWidget *pp_button;
 	/* fullscreen Play/Pause */
@@ -60,6 +67,7 @@ struct Totem {
 	GtkWidget *seek;
 	GtkAdjustment *seekadj;
 	gboolean seek_lock;
+	gboolean seekable;
 
 	/* Volume */
 	GtkWidget *volume;
@@ -85,25 +93,32 @@ struct Totem {
 	GtkAdjustment *fs_seekadj;
 	GtkWidget *fs_volume;
 	GtkAdjustment *fs_voladj;
+	GtkWidget *tcw_time_label;
 
 	guint popup_timeout;
 	gboolean popup_in_progress;
 	GdkRectangle fullscreen_rect;
 
-	ScreenSaver *scr;
+	TotemScrsaver *scr;
 
 	/* recent file stuff */
 	EggRecentModel *recent_model;
 	EggRecentViewGtk *recent_view;
 
+	/* Monitor to automatically unmount */
+	GnomeVFSVolumeMonitor *monitor;
+
+	/* session */
+	const char *argv0;
+	gint64 seek_to;
+	guint index;
+
 	/* other */
 	char *mrl;
 	TotemPlaylist *playlist;
 	GConfClient *gc;
 	TotemRemote *remote;
 	BaconMessageConnection *conn;
-	int action;
-	guint32 keypress_time;
 	TotemStates state;
 };
 
