@@ -21,7 +21,7 @@
 #define __TOTEM_H__
 
 #include <gconf/gconf-client.h>
-#include <gtk/gtkwindow.h>
+#include <gtk/gtkwidget.h>
 #include "bacon-video-widget.h"
 
 #define TOTEM_GCONF_PREFIX "/apps/totem"
@@ -38,20 +38,25 @@ void	totem_action_next			(Totem *totem);
 void	totem_action_previous			(Totem *totem);
 void	totem_action_seek_relative		(Totem *totem, int off_sec);
 void	totem_action_volume_relative		(Totem *totem, int off_pct);
-gboolean totem_action_set_mrl			(Totem *totem, const char *mrl);
+gboolean totem_action_set_mrl			(Totem *totem,
+						 const char *mrl);
+gboolean totem_action_set_mrl_with_warning	(Totem *totem,
+						 const char *mrl,
+						 gboolean warn);
 
-void	totem_action_play_media			(Totem *totem, MediaType type);
+void	totem_action_play_media			(Totem *totem,
+						 MediaType type);
 
 void	totem_action_toggle_aspect_ratio	(Totem *totem);
 void	totem_action_set_aspect_ratio		(Totem *totem, int ratio);
 int	totem_action_get_aspect_ratio		(Totem *totem);
+void	totem_action_toggle_controls		(Totem *totem);
+
 void	totem_action_set_scale_ratio		(Totem *totem, gfloat ratio);
-void    totem_action_error                      (char *title, char *reason,
+void    totem_action_error                      (const char *title,
+						 const char *reason,
 						 Totem *totem);
 
-GConfClient *totem_get_gconf_client		(Totem *totem);
-
-GtkWidget * label_create (void);
 GtkWidget * totem_statusbar_create (void);
 GtkWidget * totem_time_display_create (void);
 
